
{*******************************************************}
{                                                       }
{       Demo: Multilevel Collapsible sections           }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}

{ Version 1.2
  Required TRichView version: 17.3+ }

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ImgList, ComCtrls, RVStyle, RVScroll, RichView, RVLabelItem, RVTypes,
  CRVFData;

type
  TForm1 = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    rvDemo: TRichView;
    rvFull: TRichView;
    RVStyle1: TRVStyle;
    ImageList1: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure rvDemoJump(Sender: TObject; id: Integer);
  private
    { Private declarations }
    function IsExpanded(rv: TCustomRichView; ItemNo: Integer): Boolean;
    procedure ChangeExpanded(rv: TCustomRichView; ItemNo: Integer);    
    procedure CreateDoc;
    procedure CopyDoc;
  public
    { Public declarations }

  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ TForm1 }

{
  Creating a document.
  Important:
  - "hotspots" starts paragraphs. They are followed by LabelItems.
    LabelItems represent numbering. We cannot use "real" RichView numbering
    features because "real" numbering cannot be preceded by picture.
  - level of numbering is determinded by the paragraph style.
    Initially we create MAXLEVEL+1 paragraph styles with increasing indents.
}
const MAXLEVEL = 9;

{ Creating document in rvFull }

procedure TForm1.CreateDoc;
  { Array of list counters }
  var Numbering: array [0..MAXLEVEL] of Integer;
  { Inserting numbering }
  procedure AddNumber(Level: Integer; HasChildren: Boolean);
  var i: Integer;
      li: TRVLabelItemInfo;
      s: TRVUnicodeString;
  begin
     // Creating hypertext "minus" picture
     if HasChildren then
       rvFull.AddHotspot('', 1, 1, ImageList1, Level);
     // Updating list counters
     inc(Numbering[Level]);
     for i := Level+1 to MAXLEVEL do
       Numbering[i] := 0;
     // Calculating string to display
     s := '';
     for i := 0 to Level do
       s := s+IntToStr(Numbering[i])+'.';
     // Adding LabelItem
     li := TRVLabelItemInfo.CreateEx(rvFull.RVData, 2, s);
     li.MinWidth := -RVStyle1.ParaStyles[Level].FirstIndent;
     if HasChildren then begin
       li.MinWidth := li.MinWidth-(ImageList1.Width+2);
       li.ParaNo := -1 // adding to the same paragraph
       end
     else
       li.ParaNo := Level; // adding to new paragraph
     rvFull.AddItem('', li);
  end;

var i: Integer;
begin
   // Creating paragraphs with increasing indents
   RVStyle1.ParaStyles.Clear;
   for i := 0 to MAXLEVEL do
     with RVStyle1.ParaStyles.Add do
     begin
       FirstIndent := -(30+i*10);
       LeftIndent := i*30-FirstIndent;
     end;
   // Initializing list counters
   for i := 0 to MAXLEVEL do
     Numbering[i] := 0;
   // Building...
   AddNumber(0,True);
   rvFull.Add('Topic', 1);
     AddNumber(1,False);
     rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);
     AddNumber(1,False);
     rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);

   AddNumber(0,True);
   rvFull.Add('Topic', 1);
     AddNumber(1,True);
     rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);
       AddNumber(2,False);
       rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);
     AddNumber(1,True);
     rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);
       AddNumber(2,False);
       rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);
       AddNumber(2,False);
       rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);

   AddNumber(0,True);
   rvFull.Add('Topic', 1);
     AddNumber(1,False);
     rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);
     AddNumber(1,False);
     rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);

   rvFull.Format;
end;
{------------------------------------------------------------------------------}
{ Copying document from rvFull to rvDemo.
  Then removing collapsed sections.
}
procedure TForm1.CopyDoc;
var i,j: Integer;
    Level, StartItemNo: Integer;
begin
  // 1. Copying. AppendFrom can copy all items except for tables and controls.
  rvDemo.Clear;
  rvDemo.AppendFrom(rvFull);
  // 2. Removing collapsed sections
  // Iterating from the last item to the first item...
  for i := rvDemo.ItemCount-1 downto 0 do
    if rvDemo.IsParaStart(i) and (rvDemo.GetItemStyle(i)=rvsHotspot) and
       not IsExpanded(rvDemo, i) then
    begin // Collapsed paragraph found
      Level := rvDemo.GetItemPara(i);
      // Finding the beginning of the next paragraph
      j := i+1;
      while (j<rvDemo.ItemCount) and not rvDemo.IsParaStart(j) do
        inc(j);
      if (j=rvDemo.ItemCount) or (rvDemo.GetItemPara(j)<=Level) then
        continue;
      // Finding the next paragraph with the same or higher level (or end of doc)
      StartItemNo := j;
      while (j<rvDemo.ItemCount) and (rvDemo.GetItemPara(j)>Level) do
        inc(j);
      // Removing all between them
      rvDemo.DeleteItems(StartItemNo, j-StartItemNo);
    end;
  rvDemo.Format;
end;
{------------------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
begin
  CreateDoc;
  CopyDoc;
end;
{------------------------------------------------------------------------------}
{ Clicking hyperlink }
procedure TForm1.rvDemoJump(Sender: TObject; id: Integer);
var ItemNo, i: Integer;
    s: TRVUnicodeString;
    RVData: TCustomRVFormattedData;
begin
  // GetJumpPointItemNo can be used if you do not use tables (i.e. there are
  // no hyperlinks in table cells
  rvDemo.GetJumpPointLocation(id, RVData, ItemNo);
  // Hotspot must be followed by LabelItem
  if RVData.GetItemStyle(ItemNo+1)<>rvsLabel then
    exit;
  s := TRVLabelItemInfo(RVData.GetItem(ItemNo+1)).Text;
  // Searching rvFull for LabelItem with the same text
  for i := 0 to rvFull.ItemCount-1 do
    if (rvFull.GetItemStyle(i)=rvsLabel) and
      (TRVLabelItemInfo(rvFull.GetItem(i)).Text=s) then
    begin
      // Expanding/collasping it
      ChangeExpanded(rvFull, i-1);
      // Copying rvFull to rvDemo, taking expand/collapse state
      CopyDoc;
      exit;
    end;
end;
{------------------------------------------------------------------------------}
{ Checking the ItemNo-th item in rv. This item must be a "hotspot".
  If ImageIndex=0, it is "collapsed"
  If ImageIndex=1, it is "expanded"
}
function TForm1.IsExpanded(rv: TCustomRichView; ItemNo: Integer): Boolean;
var
  ImageIndex1,ImageIndex2: Integer;
  Tag: TRVTag;
  Text: TRVUnicodeString;
  il: TCustomImageList;
begin
  rv.GetHotspotInfo(ItemNo, Text, ImageIndex1, ImageIndex2, il, Tag);
  Result := ImageIndex1<>0;
end;
{------------------------------------------------------------------------------}
{ Changing image index the ItemNo-th item in rv (This item must be a "hotspot").
  1 <-> 0
}
procedure TForm1.ChangeExpanded(rv: TCustomRichView; ItemNo: Integer);
var
  ImageIndex1,ImageIndex2: Integer;
  Tag: TRVTag;
  Text: TRVUnicodeString;
  il: TCustomImageList;
begin
  rv.GetHotspotInfo(ItemNo, Text, ImageIndex1, ImageIndex2, il, Tag);
  if ImageIndex1=0 then
    ImageIndex1 := 1
  else
    ImageIndex1 := 0;
  rv.SetHotspotInfo(ItemNo, Text, ImageIndex1, ImageIndex1, il, Tag);
end;

end.
